from typing import Any, Dict

from ..data.domestic.request import *
from ..data.overseas.request import *
from ..service.common.base import BaseService
from ..service.common.interfaces import IChartService


class DomesticChartService(BaseService, IChartService):
    def get_minute_chart(
        self, request: DomesticMinuteChartRequest, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        endpoint = "/api/v1/quote/kr-chart/min"
        data = request.to_request_data()
        return self._request(
            "POST", endpoint, data=data, cont_yn=cont_yn, cont_key=cont_key, **kwargs
        )

    def get_daily_chart(
        self, request: DomesticDailyChartRequest, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        endpoint = "/api/v1/quote/kr-chart/day"
        data = request.to_request_data()
        return self._request(
            "POST", endpoint, data=data, cont_yn=cont_yn, cont_key=cont_key, **kwargs
        )

    def get_weekly_chart(
        self, request: DomesticWeeklyChartRequest, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        endpoint = "/api/v1/quote/kr-chart/week"
        data = request.to_request_data()
        return self._request(
            "POST", endpoint, data=data, cont_yn=cont_yn, cont_key=cont_key, **kwargs
        )

    def get_monthly_chart(
        self, request: DomesticMonthlyChartRequest, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        endpoint = "/api/v1/quote/kr-chart/month"
        data = request.to_request_data()
        return self._request(
            "POST", endpoint, data=data, cont_yn=cont_yn, cont_key=cont_key, **kwargs
        )


class OverseasChartService(BaseService, IChartService):
    def get_minute_chart(
        self, request: OverseasMinuteChartRequest, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        endpoint = "/api/v1/quote/overseas-stock/chart/min"
        data = request.to_request_data()
        return self._request(
            "POST", endpoint, data=data, cont_yn=cont_yn, cont_key=cont_key, **kwargs
        )

    def get_daily_chart(
        self, request: OverseasDailyChartRequest, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        endpoint = "/api/v1/quote/overseas-stock/chart/day"
        data = request.to_request_data()
        return self._request(
            "POST", endpoint, data=data, cont_yn=cont_yn, cont_key=cont_key, **kwargs
        )

    def get_weekly_chart(
        self, request: OverseasWeeklyChartRequest, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        endpoint = "/api/v1/quote/overseas-stock/chart/week"
        data = request.to_request_data()
        return self._request(
            "POST", endpoint, data=data, cont_yn=cont_yn, cont_key=cont_key, **kwargs
        )

    def get_monthly_chart(
        self, request: OverseasMonthlyChartRequest, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        endpoint = "/api/v1/quote/overseas-stock/chart/month"
        data = request.to_request_data()
        return self._request(
            "POST", endpoint, data=data, cont_yn=cont_yn, cont_key=cont_key, **kwargs
        )



from typing import Any, Dict

from ..data.domestic.request import *
from ..data.overseas.request import *
from ..service.common.base import BaseService
from ..service.common.interfaces import IQuoteService


class DomesticQuoteService(BaseService, IQuoteService):
    def get_stock_tickers(
        self, request: QuoteRequest, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        endpoint = "/api/v1/quote/kr-stock/inquiry/stock-ticker"
        data = request.to_request_data()
        return self._request(
            "POST", endpoint, data=data, cont_yn=cont_yn, cont_key=cont_key, **kwargs
        )

    def get_stock_price(
        self, request: QuoteRequest, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        endpoint = "/api/v1/quote/kr-stock/inquiry/price"
        data = request.to_request_data()
        return self._request(
            "POST", endpoint, data=data, cont_yn=cont_yn, cont_key=cont_key, **kwargs
        )

    def get_order_book(
        self, request: QuoteRequest, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        endpoint = "/api/v1/quote/kr-stock/inquiry/orderbook"
        data = request.to_request_data()
        return self._request(
            "POST", endpoint, data=data, cont_yn=cont_yn, cont_key=cont_key, **kwargs
        )


class OverseasQuoteService(BaseService, IQuoteService):
    def get_stock_tickers(
        self, request: OverseasStockTickersRequest, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        endpoint = "/api/v1/quote/overseas-stock/inquiry/stock-ticker"
        data = request.to_request_data()
        return self._request(
            "POST", endpoint, data=data, cont_yn=cont_yn, cont_key=cont_key, **kwargs
        )

    def get_stock_price(
        self, request: OverseasQuoteRequest, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        endpoint = "/api/v1/quote/overseas-stock/inquiry/price"
        data = request.to_request_data()
        return self._request(
            "POST", endpoint, data=data, cont_yn=cont_yn, cont_key=cont_key, **kwargs
        )

    def get_order_book(
        self, request: OverseasQuoteRequest, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        endpoint = "/api/v1/quote/overseas-stock/inquiry/orderbook"
        data = request.to_request_data()
        return self._request(
            "POST", endpoint, data=data, cont_yn=cont_yn, cont_key=cont_key, **kwargs
        )



from abc import ABC, abstractmethod
from typing import Any, Dict


class ITradingService(ABC):
    @abstractmethod
    def place_order(self, order_request, **kwargs) -> Dict[str, Any]:
        pass

    @abstractmethod
    def cancel_order(self, cancel_request, **kwargs) -> Dict[str, Any]:
        pass

    @abstractmethod
    def get_transaction_history(
        self, request, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        pass

    @abstractmethod
    def get_able_order_quantity(
        self, request, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        pass

    @abstractmethod
    def get_balance(
        self, request, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        pass

    @abstractmethod
    def get_deposit(self, cont_yn="N", cont_key=None, **kwargs) -> Dict[str, Any]:
        pass


class IQuoteService(ABC):
    @abstractmethod
    def get_stock_tickers(
        self, request, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        pass

    @abstractmethod
    def get_stock_price(
        self, request, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        pass

    @abstractmethod
    def get_order_book(
        self, request, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        pass


class IChartService(ABC):
    @abstractmethod
    def get_minute_chart(
        self, chart_request, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        pass

    @abstractmethod
    def get_daily_chart(
        self, chart_request, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        pass

    @abstractmethod
    def get_weekly_chart(
        self, chart_request, cont_yn="N", cont_key=None, **kwargs
    ) -> Dict[str, Any]:
        pass

    @abstractmethod
    def get_monthly_chart(self, cont_yn="N", cont_key=None, **kwargs) -> Dict[str, Any]:
        pass



from dataclasses import dataclass
from typing import Any, Dict, Optional

from pydbfi.data.request import *


@dataclass
class DomesticOrderRequest(OrderRequest):
    price_type: str = "00"  # 기본값: 지정가
    credit_type: str = "000"  # 기본값: 보통
    loan_date: str = "00000000"  # 기본값: 일반주문
    order_condition: str = "0"  # 기본값: 없음

    def to_request_data(self) -> Dict[str, Any]:
        if not self.stock_code.startswith("A") and len(self.stock_code) == 6:
            isu_no = self.stock_code
        else:
            isu_no = self.stock_code

        return {
            "In": {
                "IsuNo": isu_no,  # 종목번호
                "OrdQty": self.quantity,  # 주문수량
                "OrdPrc": self.price,  # 주문가
                "BnsTpCode": self.order_type,  # 매매구분
                "OrdprcPtnCode": self.price_type,  # 호가유형코드
                "MgntrnCode": self.credit_type,  # 신용거래코드
                "LoanDt": self.loan_date,  # 대출일
                "OrdCndiTpCode": self.order_condition,  # 주문조건
            }
        }


@dataclass
class DomesticCancelOrderRequest(OrderRequest):
    original_order_no: int
    stock_code: str
    quantity: int

    def to_request_data(self) -> Dict[str, Any]:
        return {
            "In": {
                "OrgOrdNo": self.original_order_no,
                "IsuNo": self.stock_code,
                "OrdQty": self.quantity,
            }
        }


@dataclass
class DomesticTransactionHistoryRequest(TransactionHistoryRequest):
    execution_status: str = "0"  # 체결여부 (0:전체, 1:체결, 2:미체결)
    order_type: str = "0"  # 매매구분 (0:전체, 1:매도, 2:매수)
    stock_type: str = "0"  # 종목구분 (0:전체)
    query_type: str = "0"  # 조회구분 (0:전체, 1:ELW, 2:ELW제외)

    def to_request_data(self) -> Dict[str, Any]:
        return {
            "In": {
                "ExecYn": self.execution_status,
                "BnsTpCode": self.order_type,
                "IsuTpCode": self.stock_type,
                "QryTp": self.query_type,
            }
        }


@dataclass
class DomesticAbleOrderQuantityRequest(AbleOrderQuantityRequest):
    order_type: str = "0"  # 매매구분 (1:매도, 2:매수)
    stock_code: str = "0"  # 종목번호
    price: float = 0  # 주문가

    def to_request_data(self) -> Dict[str, Any]:
        return {
            "In": {
                "BnsTpCode": self.order_type,
                "IsuNo": self.stock_code,
                "OrdPrc": self.price,
            }
        }


@dataclass
class DomesticBalanceRequest(BalanceRequest):
    query_type: str = (
        "0"  # 조회구분코드 (0:전체, 1:비상장제외, 2:비상장,코넥스,kotc 제외)
    )

    def to_request_data(self) -> Dict[str, Any]:
        """API 요청 데이터 형식으로 변환"""
        return {"In": {"QryTpCode": self.query_type}}


@dataclass
class DomesticQuoteRequest(QuoteRequest):
    market_type: str = (
        "J"  # 입력 조건 시장 분류 코드 (J:주식, E:ELW, EN:ETN, U:업종&지수, W:ELW)
    )
    stock_code: Optional[str] = None  # 종목 코드 (업종(U) 선택 시 지수 코드)

    def to_request_data(self) -> Dict[str, Any]:
        return {
            "In": {
                "InputCondMrktDivCode": self.market_type,
                "InputIscd1": self.stock_code,
            }
        }


@dataclass
class DomesticMinuteChartRequest(ChartRequest):
    market_type: str = "J"
    adjust_price_yn: str = "0"  # 수정 주가 사용 여부 (0:사용, 1:미사용)
    stock_code: str = "0"  # 종목 코드
    start_date: str = "0"  # 조회일자 (YYYYMMDD)
    time_interval: Optional[str] = None  # 시간 간격 (60*N: N분)

    def to_request_data(self) -> Dict[str, Any]:
        return {
            "In": {
                "InputCondMrktDivCode": self.market_type,
                "InputOrgAdjPrc": self.adjust_price_yn,
                "InputIscd1": self.stock_code,
                "InputDate1": self.start_date,
                "InputDivXtick": self.time_interval,
            }
        }


@dataclass
class DomesticDailyChartRequest(ChartRequest):
    market_type: str = "J"
    adjust_price_yn: str = "0"  # 수정 주가 사용 여부 (0:사용, 1:미사용)
    stock_code: str = "0"  # 종목 코드
    start_date: str = "0"  # 조회일자 (YYYYMMDD)
    end_date: str = "0"  # 조회일자 (YYYYMMDD)

    def to_request_data(self) -> Dict[str, Any]:
        return {
            "In": {
                "InputCondMrktDivCode": self.market_type,
                "InputOrgAdjPrc": self.adjust_price_yn,
                "InputIscd1": self.stock_code,
                "InputDate1": self.start_date,
                "InputDate2": self.end_date,
            }
        }


@dataclass
class DomesticWeeklyChartRequest(ChartRequest):
    market_type: str = "J"
    adjust_price_yn: str = "0"  # 수정 주가 사용 여부 (0:사용, 1:미사용)
    stock_code: str = "0"  # 종목 코드
    start_date: str = "0"  # 조회일자 (YYYYMMDD)
    end_date: str = "0"  # 조회일자 (YYYYMMDD)
    period: str = "W"  # 주기 (W:주, M:월, Y:년)

    def to_request_data(self) -> Dict[str, Any]:
        return {
            "In": {
                "InputCondMrktDivCode": self.market_type,
                "InputOrgAdjPrc": self.adjust_price_yn,
                "InputIscd1": self.stock_code,
                "InputDate1": self.start_date,
                "InputDate2": self.end_date,
                "InputPeriodDivCode": self.period,
            }
        }


@dataclass
class DomesticMonthlyChartRequest(ChartRequest):
    market_type: str = "J"
    adjust_price_yn: str = "0"  # 수정 주가 사용 여부 (0:사용, 1:미사용)
    stock_code: str = "0"  # 종목 코드
    start_date: str = "0"  # 조회일자 (YYYYMMDD)
    end_date: str = "0"  # 조회일자 (YYYYMMDD)
    period: str = "M"  # 주기 (W:주, M:월, Y:년)

    def to_request_data(self) -> Dict[str, Any]:
        return {
            "In": {
                "InputCondMrktDivCode": self.market_type,
                "InputOrgAdjPrc": self.adjust_price_yn,
                "InputIscd1": self.stock_code,
                "InputDate1": self.start_date,
                "InputDate2": self.end_date,
                "InputPeriodDivCode": self.period,
            }
        }



import logging
import random
import time
from typing import Any, Dict, Optional

import requests

from ...oauth import OAuth


class BaseService:
    BASE_URL = "https://openapi.db-fi.com:8443"

    def __init__(self, auth: OAuth):
        self.auth = auth
        self.logger = logging.getLogger(__name__)

    def _request(
        self,
        method: str,
        endpoint: str,
        params: Optional[Dict[str, Any]] = None,
        data: Optional[Dict[str, Any]] = None,
        headers: Optional[Dict[str, str]] = None,
        content_type: str = "application/json",
        cont_yn: str = "N",
        cont_key: str = None,
        max_retries: int = 3,
        initial_backoff: float = 1.0,
        max_backoff: float = 10.0,
        **kwargs,
    ) -> dict:
        url = f"{self.BASE_URL}{endpoint}"

        request_headers = self.auth.get_auth_header()
        request_headers["Content-Type"] = content_type

        if cont_yn:
            request_headers["cont_yn"] = cont_yn

        if cont_key:
            request_headers["cont_key"] = cont_key

        if headers:
            request_headers.update(headers)

        retry_count = 0
        backoff = initial_backoff

        while True:
            try:
                self.logger.debug(
                    f"Request to {url}, method={method}, headers={request_headers}"
                )
                self.logger.debug(f"Request data: {data}")

                if content_type == "application/json":
                    response = requests.request(
                        method=method,
                        url=url,
                        params=params,
                        json=data,
                        headers=request_headers,
                    )
                else:
                    response = requests.request(
                        method=method,
                        url=url,
                        params=params,
                        data=data,
                        headers=request_headers,
                    )

                if 500 <= response.status_code < 600:
                    if retry_count < max_retries:
                        retry_count += 1
                        jitter = random.uniform(0, 0.1 * backoff)
                        sleep_time = min(backoff + jitter, max_backoff)

                        self.logger.warning(
                            f"서버 오류 발생 (상태 코드: {response.status_code}). "
                            f"{retry_count}/{max_retries} 재시도 중... {sleep_time:.2f}초 후 재시도"
                        )

                        time.sleep(sleep_time)
                        backoff = min(backoff * 2, max_backoff)
                        continue
                    else:
                        self.logger.error(
                            f"최대 재시도 횟수({max_retries})를 초과했습니다. "
                            f"서버 오류가 지속됩니다. 상태 코드: {response.status_code}"
                        )

                response.raise_for_status()

                self.logger.debug(f"Response status: {response.status_code}")
                self.logger.debug(f"Response headers: {response.headers}")

                if "application/json" in response.headers.get("Content-Type", ""):
                    return response.json()

                return {"text": response.text}

            except requests.RequestException as e:
                if not (
                    hasattr(e, "response")
                    and e.response
                    and 500 <= e.response.status_code < 600
                ):
                    self.logger.error(f"API request failed: {str(e)}")
                    if hasattr(e, "response") and e.response is not None:
                        self.logger.error(f"Response: {e.response.text}")
                    raise

                if hasattr(e, "response") and e.response:
                    response = e.response
                    if retry_count < max_retries:
                        retry_count += 1
                        jitter = random.uniform(0, 0.1 * backoff)
                        sleep_time = min(backoff + jitter, max_backoff)

                        self.logger.warning(
                            f"서버 오류 발생 (상태 코드: {response.status_code}). "
                            f"{retry_count}/{max_retries} 재시도 중... {sleep_time:.2f}초 후 재시도"
                        )

                        time.sleep(sleep_time)
                        backoff = min(backoff * 2, max_backoff)
                        continue

                self.logger.error(f"API request failed: {str(e)}")
                if hasattr(e, "response") and e.response is not None:
                    self.logger.error(f"Response: {e.response.text}")
                raise



from dataclasses import dataclass
from typing import Any, Dict


@dataclass
class OrderRequest:
    stock_code: str
    quantity: int
    price: float
    order_type: str  # "1": 매도, "2": 매수

    def to_dict(self) -> Dict[str, Any]:
        return {k: v for k, v in self.__dict__.items() if v is not None}


@dataclass
class TransactionHistoryRequest:
    pass


@dataclass
class BalanceRequest:
    pass


@dataclass
class AbleOrderQuantityRequest:
    pass


@dataclass
class QuoteRequest:
    pass


@dataclass
class ChartRequest:
    pass
